package state51::Job::ActivateRoleWithPassword;
use Moose;
use MooseX::Types::Moose qw/ Str Int /;
use Moose::Util::TypeConstraints;
use MooseX::ClassAttribute;
use state51::Logger;
use state51::Logger::Event::SignupStateChange;
use state51::Logger::Event::PasswordReset;
extends 'state51::Job';

class_has documentation => (
    is  => 'ro',
    isa => 'Str',
    default =>
'This job is called to set the password on a new role and update the status to live.',
);

sub exchange_name {
    'activate_role_with_password';
}

has 'token' => (
    is            => 'ro',
    isa           => 'Str',
    required      => 1,
    documentation => 'The secret token authenticating the request.',
);

has 'role_id' => (
    is            => 'ro',
    isa           => 'Int',
    required      => 1,
    documentation => 'The id of the role to be activated.',
);

has 'role_password' => (
    is            => 'ro',
    isa           => 'Str',
    required      => 1,
    documentation => 'The new password that will be set on the role.',
);

has 'role_activation_email_template_id' => (
    is       => 'ro',
    isa      => 'Int',
    required => 1,
    documentation =>
'The full instance id of the email template to use for the activation email.',

   # e.g. "well done, your password has been updated. Please login for more fun"
);

sub suppress_console_display { 1 }

sub run {
    my ($self) = @_;
    state51::Base->connect;

    my $role = state51::Role->from_token( $self->token );
    die "couldn't role from_token" unless $role;
    die "role id isn't right" unless $role->id == $self->role_id;

    $role->password($self->role_password);
    $role->status('live');
    $role->update();

    state51::Logger->new->message(
        level   => 'trace',
        message => [
            state51::Logger::Event::PasswordReset->new(
                {
                    role_id => $self->role_id,
                }
            )
        ]
    );

    $self->send_password_reset_mail($role,
        $self->role_activation_email_template_id);

    # if we rep a new org then this signup is progressing
    my @new_orgs = grep {$_->status eq 'new'} @{$role->organisation_reps};
    if (@new_orgs) {
        my $new_org = $new_orgs[0];

        state51::Logger->new->message(
            level   => 'trace',
            message => [
                state51::Logger::Event::SignupStateChange->new(
                    current_value   => 'live',
                    organisation_id => $new_org->id,
                    role_id         => $role->id,
                    bounce => $role->secret.'a'.$role->uniq,
                )
            ]
        );
    }

    return 1;
}

sub send_password_reset_mail {
    my ($self, $role, $email_template_id) = @_;

    my $mail_from = 'no-reply@state51.com';

    # original reset template id : 85786987170354
    my $mail = state51::Mail::Send->new(template_id => $email_template_id);
    $mail->mail(
        {   to       => $role->email,
            from     => $mail_from,
            subject  => 'Your Password has been updated.',
            content_type => 'text/html; charset="UTF-8"',
            content_transfer_encoding => 'quoted-printable',
            bounce    => $role->secret . 'a' . $role->uniq. '@bounces.openimp.com',
            p        => {
                role      => $role,
                mail_from => $mail_from,
            },
        }
    );
}

with qw/
  state51::Mixin::JobLogger
  /;

__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;
