package state51::Job::CheckAnOrderIsReadyForDelivery;

use Data::Dumper qw/ Dumper /;
use DateTime::Duration;
use MooseX::Types::DateTime::MySQL ();
use MooseX::Types::DateTime qw/ DateTime /;
use MooseX::Types::Moose qw/ Bool /;
use Moose;
use aliased 'DateTime' => 'DT';
use state51::Base;
use state51::Logger;
use JSON qw/ encode_json /;
use namespace::autoclean;
use Sys::Hostname;
use JSON qw/ encode_json /;
use Scalar::Util qw/blessed/;

extends 'state51::Job';

sub exchange_name { 'order_delivery_checking' }

has report_on_transforms => (
    is            => 'ro',
    isa           => Bool,
    default       => 0,
    documentation => 'Report on the number of transforms remaining into RT',
);

has remove_non_exportable_products => (
    is      => 'ro',
    isa     => Bool,
    default => 0,
    documentation =>
      'Force the removal of all non-exportable products from the order',
);

has retry => (
    is      => 'ro',
    isa     => Bool,
    default => sub { shift->remove_non_exportable_products },
    lazy    => 1,
    documentation =>
      'Retry testing an order which has already failed to be deliverable',
);

sub run {
    my ($self) = @_;
    my $order = $self->_order_object;
    return 1 unless $order->status eq "ready";

    my $lock_suffix = $self->remove_non_exportable_products ? '_FORCE' : '';
    my $lockname =
      "claim_check_all_exportable_" . $self->_order_object->id . $lock_suffix;
    state51::Base->connect;
    $self->do_whilst_locked(
        $lockname,
        sub {
            if ($self->_order_object->can_order_be_marked_complete) {
                my $my_hostname = (split('\.', Sys::Hostname::hostname()))[0];
                $order->complete_order('automatic delivery script: CheckAnOrderIsReadyForDelivery.',
                    $my_hostname
                );
                return 1;
            }

# Note - this job is run regularly and is _also_ responsible for re-requesting deliveries if the
#        delivery jobs are completed, DO NOT EXIT EARLY.
            my $changed_state = 0;
            my $has_error     = 0;

            my $message =
              sprintf("Automatic checking of order %s ( %s ) found that:\n\n",
                $order->id, $order->ci_support_uri);
            if (!$order->deliver_desires_generated) {
                $message .= sprintf(
                    "Deliver desires have not been generated yet, cannot check"
                );
                $changed_state = 1;
            }
            else {
                if (!defined($order->no_of_transforms_requested)) {
                    $order->no_of_transforms_requested(0);
                }

                if (!$order->all_transforms_completed) {
                    my @incomplete_transforms;
                    if ($order->no_of_transforms_requested > 0) {
                        @incomplete_transforms =
                          grep { $_->status ne "junk" }
                          state51::FileTransform->fetch(
                            to_fulfil  => $order,
                            not_status => 'complete'
                          );
                    }
                    if ($order->transforms_requested) {
                        if (scalar(@incomplete_transforms) == 0) {
                            if ($self->assert_all_transforms_are_done($order)) {
                                $message .= sprintf(
"All %s transforms requested have been completed\n",
                                    $order->no_of_transforms_requested);
                                $order->all_transforms_completed(
                                    state51::Base->dbnow);
                                $order->no_problem_transforms(0);
                                $order->no_failed_transforms(0);
                                $changed_state = 1;
                            }
                            else {
                                $message .= sprintf(
"%s transforms requested have been completed, yet still there are outstanding encodes\n",
                                    $order->no_of_transforms_requested);
                            }
                        }
                        else {
                            my $total      = $order->no_of_transforms_requested;
                            my $incomplete = scalar @incomplete_transforms;
                            my $complete   = $total - $incomplete;
                            my $problem    = scalar(
                                state51::FileTransform->fetch_readonly(
                                    to_fulfil => $order,
                                    status    => 'problem',
                                )
                            );
                            $problem += scalar(
                                state51::FileTransform->fetch_readonly(
                                    to_fulfil => $order,
                                    status    => 'unfixable',
                                )
                            );

                            my $failed = scalar(
                                state51::FileTransform->fetch_readonly(
                                    to_fulfil => $order,
                                    status    => 'failed',
                                )
                            );
                            $message .=
"Of $total transforms requested, $complete have been completed and $incomplete are incomplete.\n\tThere are $failed temporary encodes issues and $problem permanent encode issues\n";
                            $order->no_problem_transforms($problem);
                            $order->no_failed_transforms($failed);

             # NOTE - We _do not_ change state here, so this doesn't write into
             #        RT unless we just filled in number_of_transforms_requested
             #        above..a
                            $changed_state = 1 if $self->report_on_transforms;
                        }
                    }
                    else {
                        require state51::Job::RequestTransformsForOrder;
                        my $order_id = $order->id;    # Jobs unload!
                        state51::Job::RequestTransformsForOrder->new(
                            order => $self->order)->run;
                        $order = state51::Request::Export->load($order_id);
                    }
                }

                # Test all products in the order look deliverable step
                if (!$order->all_formats_exportable) {
                    my (
                        $errors_count,  $errors_report,
                        $error_desires, $exportable_desires
                      )
                      = $order->check_not_exportable_formats(
                        "CheckAnOrderIsReadyForDelivery");

                    # Could take ages, avoid race conditions.
                    $order = state51::Base->storage->reload($order);
                    return unless $order->status eq "ready";
                    if ($errors_count == 0)
                    {    # Has set $order->all_formats_exportable
                        $message .=
"All products are exportable, delivery can start now.\n";
                        $order->order_failed_to_be_deliverable(undef);
                    }
                    elsif ($self->remove_non_exportable_products) {
                        foreach (@$error_desires) {
                            $_->update;
                        }
                        $message .= sprintf(
"Marked as undeliverable %d not exportable deliver desires from the order.\nDelivery FORCED TO START\n\nErrors  found:\n%s",
                            scalar @$error_desires,
                            $errors_report
                        );
                        $order->order_failed_to_be_deliverable(undef);
                        $order->all_formats_exportable(state51::Base->dbnow);
                    }
                    else {
                        $message .= sprintf(
"Has %d not exportable products, check errors below:\n\n%s",
                            $errors_count, $errors_report);
                        $has_error = 1;
                    }
                    $changed_state = 1;
                }
            }
            $order->last_all_formats_exportable_check(state51::Base->dbnow);
            $order->update;

            # DO NOT MOVE THIS DOWN OTHERWISE WE STOP FIRING IT
            if ($order->all_formats_exportable) {
                $self->run_job('state51::Job::KickOffDeliveriesForOrder',
                    order => $self->order);
            }

            return 1 unless $changed_state;

            eval { $self->update_ticket_status('open') if $has_error; };
            if ($@) {
                $message .=
"\n\nCheckAnOrderIsReadyForDelivery:---------------\nERROR UPDATE_TICKET_STATUS IN RT $_";
                $self->log_message([$message]);
            }
        }
    );
    return 1;
}

sub assert_all_transforms_are_done {
    my ($self, $order) = @_;

    my @errors;

    foreach my $ms_org (@{$order->musicservice_destinations()}) {
        next
          if $ms_org->development_status =~ /^(dormant|archived)$/
          ; # Probably can't construct any more as the code will be deleted! Found in old auto-exports..
        my $ms = $ms_org->musicservice_class_instance(
            export_request => $order,
            option_dry_run => 0,
        );

        foreach my $dd (@{$order->get_living_delivery_desires_to_dp($ms_org)}) {
            eval { $ms->is_format_audio_available(format => $dd->asset); };
            if ($@) {
                push @errors, $_;
                $self->log_message(
                    [
                            "Order "
                          . $self->_order_object->id
                          . " missing encodes after encodes complete: $_"
                    ]
                );

            }
        }
    }

    if (@errors) {
        $self->log_message(
            [
                    "Order "
                  . $self->_order_object->id
                  . " has errors:\n\n"
                  . join("\n", @errors)
            ]
        );
        require state51::Job::RequestTransformsForOrder;
        $self->log_message(
            [
                state51::Job::RequestTransformsForOrder->new(
                    order => $self->order
                )
            ]
        );
        return;
    }
    return 1;
}

with qw/
  state51::Mixin::HandleOrdersOnJobs
  state51::Mixin::RunsAsyncJobs
  state51::Mixin::JobSignature
  state51::Mixin::HasLocks
  /;

__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;
