package state51::Job::CreateAlbumZip;
use state51::Moose;
use MooseX::Storage;
with Storage( format => "JSON" );

use MooseX::Types::Moose qw/ Int /;
use File::Temp qw/ tempdir /;
use Encode qw/ encode /;
use List::MoreUtils qw/ uniq /;
use XML::XSPF;
use Cwd qw/ getcwd /;
use Data::Dump qw/ dump /;

extends 'state51::Job';

# We make these just in time, so we need to do them fast
sub exchange_name {
    'run_encode_background'
}

sub suppress_console_display { 0 }

has id => (
    isa      => Int,
    is       => 'ro',
    required => 1,
);

sub clean_filename {
    my ( $self, $fn, $ext ) = @_;
    my $out = encode( "utf-8", $fn );
    $out =~ s/\.|\/|\\//g;
    $out ||= "x";
    die "could not make a filename!" if length($out) < 1;
    if ($ext) {
        return "$out.$ext";
    }
    return $out;
}

method run() {
    state51::Base->connect;

    $self->do_whilst_locked(
        "create_album_zip_" . $self->id,
        sub {
            my $zr = state51::ZipRequest->load( $self->id );
            if ( !$zr->file || $zr->file->status ne 'live' ) {
                my $rf = $zr->asset;

                my $td = tempdir( TMPDIR => 1 );
                my $title =
                    $rf->can('title_with_version')
                  ? $rf->title_with_version
                  : $rf->title;
                my $album = "$td/" . $self->clean_filename($title) . "/";
                mkdir($album) or die "could not mkdir '$album': $!";

                my @xspf_tracks;

                open( my $m3ufh, ">",
                    $album . $self->clean_filename( $title, "m3u" ) )
                  or die "could not open m3u: $!";

                foreach my $t ( @{ $rf->ordered_tracks } ) {
                    my ($audio) = grep { $_->status eq 'live' }
                      @{ $t->audio_with_encoding( $zr->audio_encoding ) };
                    die "No live audio in TRACK: "
                      . dump( $t->audio_with_encoding() )
                      . "ZIP REQUEST: "
                      . dump( $zr->audio_encoding )
                      unless defined $audio;

                    my $fn = $self->clean_filename(
                        sprintf( "%02d-%03d-%s",
                            $t->volume_part, $t->track_number,
                            $t->label_copy_with_mix ),
                        $audio->extension,
                    );
                    $audio->download_to( $album . $fn );
                    my $xspf_track = XML::XSPF::Track->new;
                    $xspf_track->title( $t->label_copy_with_mix );
                    $xspf_track->creator( $t->display_artist );
                    $xspf_track->duration( $t->duration );
                    $xspf_track->trackNum( $t->track_number );
                    $xspf_track->location($fn);
                    push @xspf_tracks, $xspf_track;
                    print $m3ufh "$fn\n";
                }
                close $m3ufh;

                my @artworks = uniq grep { $_->status eq 'live' }
                  ( $rf->packshot, @{ $rf->artworks } );

                foreach my $file (@artworks) {
                    $file->download_to(
                        $album
                          . $self->clean_filename(
                            ( $file->caption || $file->id ),
                            $file->extension,
                          )
                    );
                }

                my $xspf = XML::XSPF->new();
                $xspf->image(
                    $self->clean_filename(
                        ( $artworks[0]->caption || $artworks[0]->id ),
                        $artworks[0]->extension,
                    )
                );
                $xspf->title($title);
                if ( $rf->can('display_artist') ) {
                    $xspf->creator( $rf->display_artist );
                }
                $xspf->trackList(@xspf_tracks);
                open( my $fh, ">",
                    $album . $self->clean_filename( $title, "xspf" ) )
                  or die "could not open xspf: $!";
                print $fh $xspf->toString();
                close $fh;

                my $old_dir = getcwd();
                chdir($td);
                my $zip_rc = system( "zip", "-r", "-9", "album.zip",
                    $self->clean_filename($title) );
                chdir($old_dir);
                die "zip failed" if $zip_rc != 0;

                my $zip = state51::File->new_from_file(
                    "$td/album.zip",
                    source           => $rf->source,
                    asset_controller => $rf->asset_controller,
                    encoded_by_us    => 1,
                );
                warn "path : " . $zip->local_file_path;
                $zip->cache('FORCE');
                warn "path : " . $zip->local_file_path;
                $zip->caption($title);
                $zip->write();
                $zr->file($zip);
                $zr->update;
            }

            foreach
              my $fk ( state51::FileKey::ZippedAlbum->fetch( context => $zr ) )
            {
                $fk->file( $zr->file );
                $fk->update;
            }
        },
    );
}

with qw/
  state51::Mixin::HasLocks

  state51::Mixin::Loggable
  state51::Mixin::UnloadAfterJob
  /;

__PACKAGE__->meta->make_immutable;
1;

