package state51::Job::CreateDeliverDesiresForXls;
use state51::Base;
use state51::Moose;
use MooseX::Storage;
with Storage (format => "JSON");

use Try::Tiny;
use MooseX::Types::Moose qw/ Int Bool Undef/;
use MooseX::Types::Common::String qw/ NonEmptySimpleStr /;
use namespace::autoclean;

extends 'state51::Job';

has file_no => (
    is => 'ro',
    isa => Int|Undef,
    documentation => 'The uploaded XLS file number to be validated',
    lazy => 1,
    builder => '_build_file_no',
);

method _build_file_no() {
    my $file_no = undef;
    try { $file_no = $self->_order_object->_find_order_xls_file_number; };
    return $file_no;
}

has ignore_validation_errors => (
    is => 'ro',
    isa => Bool,
    default => 1,
    documentation => 'Ignore spreadhseet validation errors, just generate the known good products',
);

method run() {
    $0 = 'perl jobserver_worker: CreateDeliverDesiresForXls for ' . $self->order;
    my (@jobs, $errors, $product_count, $track_count, $correspondence, $comments, $order_status);
    my $request = $self->_order_object;
    my $ticket_status = 'stalled';
    my $exception = 0;
    my $validated = !$request->require_xls_validation;
    my $had_desires = $request->deliver_desires_generated;
    if ($had_desires) {
        print "Order " . $request->id . " already had desires generated, skipping\n";
    }
    try {
        if (defined $self->file_no) {
            ($errors, $product_count, $track_count) = $request->order_xls_to_deliver_desire($self->file_no, $self->ignore_validation_errors);
            $request = state51::Base->storage->reload($request);
            if(not $request->deliver_desires_generated and $product_count){
                $request->deliver_desires_generated(state51::Base->dbnow);
                $request->update;
            }
            if (scalar(keys %$errors) && $self->ignore_validation_errors ) {
                if ($product_count) {
                    push(@jobs, ['state51::Job::ValidateDeliverDesires', order => $self->order]);
                    $validated = 1;
                    $comments  .= "\nValidate Desires Job sent";
                }
            }
            elsif (not scalar(keys %$errors) ){
                push(@jobs, ['state51::Job::ValidateDeliverDesires', order => $self->order]);
                $validated = 1;
                $comments  .= "\nValidate Desires Job sent";
            }
        }
        elsif($had_desires) {
            push(@jobs, ['state51::Job::ValidateDeliverDesires', order => $self->order]);
            $validated = 1;
            $comments  .= "\nValidate Desires Job sent";
        }
    }
    catch {
        $comments .= "\n\nException validating XLS and generating Delivery Desires (order id " . $self->order . " file_no " . $self->file_no . "): $_\n";
        $exception = 1;
    };

    return unless $request->status eq "validating";

    if ($validated) {
        $order_status = 'validating';
        if (scalar(keys %$errors)) { #this just happen when $self->ignore_validation_errors == 1
            $correspondence = "Not all the products and services listed in the ".
                "spreadsheet have been found, but the order will proceed with the ".
                "validation of remaining products found ($product_count products, ".
                "$track_count tracks). \n\nErrors found are listed below and recorded ".
                "on the order's notes field for future reference: ".
                $request->describe_validation_errors($errors);
            $request->append_notes($correspondence);
            $request->update;
        }
    }
    elsif($exception == 0) {
        $order_status = 'pending';
        $correspondence = sprintf(
            "Please find below the automated report on the latest spreadsheet(s) uploaded for delivery order %s.\n\n".
            "Please now amend the delivery spreadsheet for this order by removing the products and services noted below ".
            "and then re-upload it to the order page here:\n%s\n".
            qq{After doing so, hit the "Post Order" button.\nThis delivery order will then be able to be processed.},
            $request->id,
            $request->ci_support_uri,
        );

        if (scalar(keys %$errors)) {
            $correspondence .= "\n\nDelivery spreadsheet errors: ".$request->describe_validation_errors($errors);
        }
        elsif ($request->require_xls_validation && not defined $self->file_no ){
            $correspondence .= "\n\nThere is no spreadsheet attached to the order. Please attach one including details of the deliveries desired and re-post the order";
        }
    }
    elsif($exception == 1) {
        $ticket_status = 'open';
    }

    if ($exception) {
        $comments = sprintf("Files attached processed at %s but an exception was thrown operating on delivery file number %s (%s errors)\n%s \n\nPlease fix them and retry a validation or report back to the client\n",
            ($self->can('date') ? $self->date : 'NO_DATE'),
            (defined $self->file_no)? $self->file_no:'n/a',
            (defined $errors)?scalar(keys %$errors):'n/a',
            $comments||''
        );
    }
    else {
        try {
            $request->_set_status($order_status);
            $request->update;
        } catch {
            $comments .= "\nError updating status:\n".$_;
            $ticket_status = 'open';
        };

        $comments = sprintf("Delivery Desires have been generated at %s operating on delivery file number %s (%s errors), deliver desire validation should proceed next\n%s",
            $request->deliver_desires_generated,
            (defined $self->file_no)? $self->file_no:'n/a',
            (defined $errors)?scalar(keys %$errors):'n/a',
            $comments||''
        );
    }

    try {
        $self->correspond_in_rt($correspondence) if ($correspondence && not $had_desires);
        $self->update_ticket_status($ticket_status);
    }
    catch {
        $comments .= "ERROR COMMENTING IN RT $_";
        warn("CreateDeliverDesiresForXls:" . $comments);
    };
    print $comments if $self->interactive;
    if ($validated) {
        # If all music services in this order are testing, make the order free
        my @music_services = $self->_order_object->musicservice_destinations->flatten;
        my $can_be_free = 1;
        foreach my $s (@music_services) {
            $can_be_free = 0 if $s->development_status ne "testing";
        }
        $self->_order_object->do_not_charge($can_be_free);
        $self->_order_object->update;

        # Go on to run jobs
        $self->run_job(@$_) for @jobs;
    }
    return 1;
}

method lockname() { "order_generate_dd_from_xls_".$self->order }

with qw/
    state51::Mixin::Loggable
    state51::Mixin::HandleOrdersOnJobs
    state51::Mixin::RunsAsyncJobs
    state51::Mixin::JobSignature
    state51::Script::Helper::InteractiveSuccess
    state51::Mixin::LockedJob
/;

#
# __PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;
