package state51::Job::CreateLegalNaturalPerson;
use Moose;
use MooseX::Types::Moose qw/ Str Int /;
use Moose::Util::TypeConstraints;
use MooseX::ClassAttribute;
use state51::Logger;
use state51::Logger::Event::SignupStateChange;
extends 'state51::Job';

class_has documentation => (
    is  => 'ro',
    isa => 'Str',
    default =>
'Called to create a Legal or Natural Person and associate them with an existing role and org.',
);

sub exchange_name {
    'create_legal_natural_person';
}

has 'role_id' => (
    is            => 'ro',
    isa           => 'Int',
    required      => 1,
    documentation => 'The id of the role.',
);

has 'organisation_id' => (
    is            => 'ro',
    isa           => 'Int',
    required      => 1,
    documentation => 'The id of the organisation.',
);

has 'person_type' => (
    is            => 'ro',
    isa           => enum([qw/ Legal Natural /]),
    required      => 1,
    documentation => 'The type of person to be created.',
);

has 'person_name' => (
    is            => 'ro',
    isa           => 'Str',
    required      => 1,
    documentation => 'The name of the person to be created.',
);

has 'person_address_address' => (
    is            => 'ro',
    isa           => 'Str',
    required      => 1,
    documentation => 'The address of the person to be created.',
);

has 'person_address_postcode' => (
    is            => 'ro',
    isa           => 'Str',
    required      => 1,
    documentation => 'The postcode of the person to be created.',
);

has 'person_address_country_iso_code_2' => (
    is       => 'ro',
    isa      => 'Str',
    required => 1,
    documentation =>
      'The 2 letter iso code of the country for the person to be created.',
);

has 'person_website' => (
    is            => 'ro',
    isa           => 'Str',
    required      => 0,
    documentation => 'The website of the person to be created.',
);

has 'person_other_social_urls' => (
    is            => 'ro',
    isa           => 'ArrayRef[Str]',
    required      => 1,
    documentation => 'The other social URLs of the person to be created.',
);

has 'person_opencorporates_url' => (
    is       => 'ro',
    isa      => 'Str',
    required => 0,
    documentation =>
      'The opencorporates URL of the person legal to be created.',
);

has 'person_corporate_registration_id' => (
    is       => 'ro',
    isa      => 'Str',
    required => 0,
    documentation =>
      'The corporate registration id of the person legal to be created.',
);

has 'person_person_name' => (
    is       => 'ro',
    isa      => 'Str',
    required => 0,
    documentation =>
'The name of the natural person associated with the legal person to be created.',
);

sub suppress_console_display { 1 }

sub run {
    my ($self) = @_;
    state51::Base->connect;

    # it's a subclass of org but we don't know which one
    my $typeless_org_id = substr($self->organisation_id, 0, -4);
    my $organisation = state51::Organisation->load_polymorphic($typeless_org_id);

    die "org ".$organisation->id." has wrong source "
      unless (($organisation->source->id eq '85788250840009') or ($organisation->source->id eq '85788250850009'));

    my $address = $self->make_address(
        $self->person_address_address,
        $self->person_address_postcode,
        $self->person_address_country_iso_code_2,
        $organisation->source,
    );

    my @social_urls = $self->make_social_urls($organisation->source, $self->person_other_social_urls);
    my $regular_url = $self->make_regular_url($organisation->source, $self->person_website);

    $self->make_person(
        $self->person_type,
        $self->person_name,
        $address,
        $self->person_opencorporates_url,
        $self->person_corporate_registration_id,
        $self->person_person_name,
        $organisation,
        $self->role_id,
    );

    my @usable_urls = grep {$_} ($regular_url, @social_urls);
    if (@usable_urls) {
        $organisation->website(\@usable_urls);
    }
    $organisation->status('compliance');
    $organisation->update;

    state51::Logger->new->message(
        level   => 'trace',
        message => [
            state51::Logger::Event::SignupStateChange->new(
                current_value   => 'compliance',
                organisation_id => $organisation->id,
                role_id         => $self->role_id,
            )
        ]
    );

    return 1;
}

sub make_address {
    my ($self, $address, $postcode, $country_iso_code_2, $source) = @_;

    my $country = state51::Country->fetch_one(iso_code2 => $country_iso_code_2);

    my $address_instance = state51::Address->fetch_or_create(
        address => $address,
        address_postcode => $postcode,
        country => $country,
        source => $source,
    );
    $address_instance->upsert();

    return $address_instance;
}

sub make_regular_url {
    my ($self, $source, $url) = @_;

    my $regular_url;
    if ($url) {
       $regular_url = state51::Uri->fetch_or_create(
          uri    => $url,
          source => $source,
          status => 'live',
          type   => 'homepage'
      );
      $regular_url->upsert();
    }

    return $regular_url;
}

sub make_social_urls {
    my ($self, $source, $social_urls) = @_;

    my @urls;
    foreach my $url (@{$social_urls}) {
        my $social_url = state51::Uri->fetch_or_create(
            uri    => $url,
            source => $source,
            status => 'live',
            type   => 'social'
        );
        $social_url->upsert();

        push @urls, $social_url;
    }

    return @urls;
}

sub make_person {
    my ($self, $person_type, $person_name, $address, $opencorporates_url, $corporate_registration_id, $person_person_name, $organisation, $role_id) = @_;

    my $role = state51::Role->load($role_id);

    my $person;
    my $aegis;
    if ($person_type eq 'Legal') {
        $aegis = state51::Person::Legal->fetch_or_create(
            name => $person_name,
            registered_address => $address,
            corporate_registration_id => $corporate_registration_id,
            opencorporates_url => $opencorporates_url,
        );
        $person = state51::Person::Natural->fetch_or_create(
            name => $person_person_name,
            registered_address => $address,
        );
    }
    else {
        $person = state51::Person::Natural->fetch_or_create(
            name => $person_name,
            registered_address => $address,
        );
        $aegis = $person;
    }

    $person->upsert();

    $role->person($person);
    $role->update();

    $organisation->aegis($aegis);
    $organisation->update();

    return $person;
}

with qw/
  state51::Mixin::JobLogger
  /;

__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;
