package state51::Job::CreateRoleAndOrgForSignup;
use Moose;
use MooseX::Types::Moose qw/ Str Int /;
use Moose::Util::TypeConstraints;
use MooseX::ClassAttribute;
use MooseX::Types::Email qw/EmailAddress/;
use state51::Logger;
use state51::Logger::Event::SignupStateChange;

extends 'state51::Job';

class_has documentation => (
    is  => 'ro',
    isa => 'Str',
    default =>
'The initial step of the signup application creates a role and organisation.',
);

sub exchange_name {
    'create_role_and_org_for_signup';
}

has 'role_name' => (
    is            => 'ro',
    isa           => 'Str',
    required      => 1,
    documentation => 'The name of the role to be created.',
);

has 'role_email' => (
    is            => 'ro',
    isa           => 'Str',
    required      => 1,
    documentation => 'The email address of the role to be created.',
);

has 'role_phone' => (
    is            => 'ro',
    isa           => 'Str',
    required      => 1,
    documentation => 'The phone number of the role to be created.',
);

has 'organisation_name' => (
    is            => 'ro',
    isa           => 'Str',
    required      => 1,
    documentation => 'The name of the organisation to be created.',
);

has 'organisation_class' => (
    is            => 'ro',
    isa           => enum([qw/ artist label distributor /]),
    required      => 1,
    documentation => 'The class of the organisation to be created.',
);

has 'source_id' => (
    is       => 'ro',
    isa           => enum(['85788250840009', '85788250850009']),
    required => 1,
    documentation =>
      'The id of the source to be set on the new organisation.',
);

has 'role_creation_callback_url' => (
    is       => 'ro',
    isa      => 'Str',
    required => 1,
    documentation =>
'The callback URL to be used in the email after creating the role and organisation.',
);

has 'role_creation_email_template_id' => (
    is       => 'ro',
    isa      => 'Int',
    required => 1,
    documentation =>
      'The full instance id of the email template to be used in response to the user.',

    # e.g. "click this link to set your password"
);

sub suppress_console_display { 1 }

sub run {
    my ($self) = @_;
    state51::Base->connect;

    my $source = state51::Source->load($self->source_id);

    # if an org already exists with this name, it's fine we still make a new one
    # and compliance will sort it out.
    my $organisation_class =
      'state51::Organisation::' . ucfirst($self->organisation_class);
    my $organisation = $organisation_class->new(
        name   => $self->organisation_name,
        source => $source,
        status => 'new',
    );
    $organisation->insert();

    die "invalid email address: ".$self->role_email unless is_EmailAddress($self->role_email);

# what if a role exists with that email?
# it's legit, it may exist in new/pending, if so then re-use and ensure it's 'new'
    my ($role) = state51::Role->fetch(
        email  => $self->role_email,
        status => [ 'new', 'pending' ]
    );
    if (not $role) {
        $role = state51::Role->new(email => $self->role_email);
    }

    $role->name($self->role_name);
    $role->phone($self->role_phone);
    $role->status('new');
    $role->source($source);

    $role->upsert;
    $role->add_organisation_rep($organisation);
    $role->update;

    # send mail containing role token (secret and role id blob) and callback url
    $role->new_secret();
    my $url =
      $self->role_creation_callback_url . '?role_id=' . $role->id . '&secret=' . $role->to_token;

    $self->send_role_creation_email(
        $role,
        $self->role_creation_email_template_id,
        $url,
        $source,
    );

    state51::Logger->new->message(
    level   => 'trace',
    message => [
        state51::Logger::Event::SignupStateChange->new(
            current_value  => 'new',
            organisation_id => $organisation->id,
            role_id         => $role->id,
        )
    ]
);

    return 1;
}

sub send_role_creation_email {
    my ($self, $role, $template_id, $url, $source) = @_;

    # original signup template id: 85786940840354
    my $mail_from = 'no-reply@state51.com';

    my $mail = state51::Mail::Send->new(template_id => $template_id);
    $mail->mail(
        {   to       => $role->email,
            from     => $mail_from,
            subject  => 'Please verify your email address',
            content_type => 'text/html; charset="UTF-8"',
            content_transfer_encoding => 'quoted-printable',
            bounce    => $role->secret . 'a' . $role->uniq. '@bounces.openimp.com',
            p        => {
                role      => $role,
                url       => $url,
                mail_from => $mail_from,
                source_name => $source->name(),
            },
        }
    );
}


with qw/
  state51::Mixin::JobLogger
  /;

__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;
