package state51::Job::DeliverOneDPForOrder;
use Moose;
use Moose::Util::TypeConstraints;
use MooseX::Types::Moose qw/ Int /;
use MooseX::Storage;
with Storage (format => "JSON");
use namespace::autoclean;

use Capture::Tiny qw/ capture_merged /;
use Data::UUID;
use Filesys::Df qw/ df /;
use IO::Handle;
use JSON qw/ encode_json /;
use Try::Tiny;
use Scalar::Util qw/ blessed /;
use MooseX::Types::Moose qw/Bool/;

use state51::Base;
use state51::MusicServiceFactory;
use state51::Job::RetryTransferForBatch;
use state51::SQLHacks qw/ are_all_dds_finished_for_this_dp /;

extends 'state51::Job';

sub exchange_name { 'orderdelivery_' . lc($_[0]->_order_object->priority) }

has 'delivery_point_id' => (
    is            => 'ro',
    isa           => Int,
    required      => 1,
    documentation => 'a DeliveryPoint ID',
);

class_type 'state51::OrganisationAttribute::MusicService';

has _delivery_point => (
    is => 'ro',
    isa => 'state51::OrganisationAttribute::MusicService',
    traits => [qw/ DoNotSerialize /],
    default => sub {
        state51::OrganisationAttribute::MusicService->load($_[0]->delivery_point_id);
    },
    lazy => 1,
    clearer => '_clear_delivery_point',
);

has self_started => (
    is  => 'ro',
    isa => Bool,
    default => 0,
    documentation => 'Were we run by KickOff or did we run ourselves?',
);

# _order_object is from state51::Mixin::HandleOrdersOnJobs

sub get_formats_to_deliver {
    my ($self, $limit) = @_;
    my $formats = $self->_order_object->get_formats_not_exported_yet_to_dp($self->_delivery_point, $limit);
    return @$formats;
}

sub can_trigger_more_delivery { 1 }

sub command_line { ' for ' . $_[0]->order . ' to ' . $_[0]->_delivery_point->tla; }

my $ug = Data::UUID->new;
has uuid => (
    is => 'ro',
    default => sub { my $uuid = $ug->create; $ug->to_string($uuid) },
);

sub run {
    my ($self) = @_;

    my $my_hostname = (split('\.',Sys::Hostname::hostname()))[0];
    unless ($self->_order_object->status eq "ready") {
        $self->log_message(level => "warn", message => ["Order object id:" . $self->order . " is status " . $self->_order_object->status . " not ready"]) if $self->interactive;
        return 1;
    }
    if ((-d "/exports") and (!$ENV{HARNESS_ACTIVE})) {
        my $df = df("/exports");
        my $max = 1024*1024*150; # Somewhat over 10% free
        if ($df->{bfree} < $max) { # Somewhat over 10% free
            $self->log_message(level => "warn", message => ["Order object id:" . $self->order . ". Aborting job as bfree on /exports " . $df->{bfree} . " < $max\n"]);
            return 1;
        }
    }

    my $comments = '';
    # order object may not have an id
    $self->_order_object->upsert;
    # can't use filter_count as there's no not_includes in
    my $work_to_do = state51::SQLHacks::are_all_dds_finished_for_this_dp($self->_order_object, $self->_delivery_point);
    if (not $work_to_do) {
        $self->log_message(level => "warn", message => ["Skipping job as order id:" . $self->order . " appears to be complete to " . $self->_delivery_point->tla]) if $self->interactive;
        if ($self->_order_object->can_order_be_marked_complete) {
            $self->log_message(level => "warn", message => ["Entire order id:" . $self->order->id . "seems to be complete, completing."]) if $self->interactive;
            $self->_order_object->complete_order('automatic delivery script, rechecked entire order : DeliverOneDPForOrder.', $my_hostname);
        }
        return 1;
    }

    if ($self->_delivery_point->development_status() eq 'on-hold') {
        $self->log_message(level => "warn", message => ["Order object id:" . $self->order . ". Skipping job as " . $self->_delivery_point->tla . " is on-hold\n"]) if $self->interactive;
        return 1;
    }

    my ($td) = state51::TransferDetails->fetch(
        destination => $self->_delivery_point,
        organisation => $self->_order_object,
        not_status => 'archived',
    );

    if ($td && $td->status eq 'on-hold') {
        $self->log_message(level => "warn", message => ["Order object id:" . $self->order . ". Skipping job as transfer details have this on-hold\n"]) if $self->interactive;
        return 1;
    }

    # in the smallest lock possible, get some work that needs doing
    my @formats;
    my $lockname = $self->claim_dds_lockname;
    my $got_a_lock = 0;
GETLOCK:
    $self->do_whilst_locked(
        $lockname,
        sub {
            $got_a_lock = 1;
            # returns things that need doing
            @formats = $self->get_formats_to_deliver($self->_delivery_point->default_batch_size || 100);
            return unless @formats;

            my @ready_and_locked_formats;
            if (!$ENV{HARNESS_ACTIVE}) {
                # ensure that the DDs aren't complete, and take an advisory lock on the ones we plan to deliver
                my @dd = state51::DeliverDesire->fetch(
                    asset   => \@formats,
                    request => $self->_order_object,
                    entity  => $self->_delivery_point,
                );
                foreach my $dd_to_claim (@dd) {
                    if ((not $dd_to_claim->is_complete) and ($dd_to_claim->status ne 'sysfail') and ($dd_to_claim->lock())) {
                        push @ready_and_locked_formats, $dd_to_claim->asset;
                        $dd_to_claim->increment_delivery_attempts();
                        $dd_to_claim->update();
                    }
                    $dd_to_claim->unload();
                }
                @formats = @ready_and_locked_formats;
            }
        }
    );
    unless ($got_a_lock) {
        sleep(120); # wait for someone else to free it up
        goto GETLOCK;
    }

    unless (scalar @formats) {
        $self->log_message(level => "warn", message => ["Order object id:" . $self->order . ". No formats found to deliver"]) if $self->interactive;
        return 1;
    }

    my $log_path = $self->get_log_path();
    my $log_fh = $self->get_log_fh($log_path);
    return $self->cannot_write_log($log_path) unless $log_fh;

    my ($needs_more, $has_error, $has_exception) = (0, 0, 0);
    my ($export_batch, $export_batch_id);
    my $out = capture_merged {
        try {
            $comments .= sprintf("%s started delivery for order: %s to: %s\n",
                localtime(), $self->order, $self->_delivery_point->tla);

            my $musicservice = state51::MusicServiceFactory->new->construct_musicservice($self->_delivery_point, {
                trace => 1,
                export_request => $self->_order_object,
                music_sources => [ $self->_order_object->organisation->media_catalogue, ],
                do_transport => $ENV{MUSIC_SERVICE_DO_TRANSPORT} || 0,
            });

            $musicservice->set_handler_defaults();
            $musicservice->exportable_test->{is_format_not_already_exported} = 1;
            $musicservice->exportable_test->{is_from_job} .= 'DeliverOneDPForOrder';
            $musicservice->option_dry_run(0); # defaults to dry run
            $musicservice->set_only_process_formats(formats => \@formats);
            if (scalar @formats) {
                $musicservice->set_only_process_formats(formats => \@formats);
                $export_batch = $musicservice->export(delivery_job_uuid => $self->uuid);
                $export_batch_id = $export_batch ? $export_batch->id : 'no batch made';
                $self->_clear_order_object; # Re-retrieve, we may have reconnected.
                $self->_clear_delivery_point;
                $comments .= "\n - Export Batch ID: $export_batch_id";
                my $sbid = ($export_batch && $export_batch->service_batch_identifier) || 'none';
                $comments .= "\n - Service Batch ID: $sbid";
                $comments .= "\n - Delivery Point ID: " . $self->delivery_point_id;
                if ($export_batch) {
                    $comments .= "\n - Batch generation status is " . $export_batch->status;
                    $comments .= "\n - Exported batch status is ".$export_batch->service_status
                        if $export_batch->service_status;
                }
                $comments .= "\n - Export path: ".$musicservice->export_path();
                $comments .= "\n - Export log path: ".$log_path;
                my $products_count = $export_batch ? scalar(@{$export_batch->export_item}) : 0;
                if (not $products_count) {
                    $comments .= "\n - Nothing exported to that musicservice";
                }
                else {
                    $comments .= "\n - $products_count formats exported successfully";
                    $self->run_job('state51::Job::DoTransferForBatch', batch_id => $export_batch_id);
                    $needs_more = 1 if ($products_count < scalar(@formats)); # Cause another round of deliveries
                }
            }
            else {
                $comments .= "Nothing found to deliver to ".$self->_delivery_point->tla;
            }
        }
        catch {
            $comments .= "CAUGHT EXCEPTION: $_";
            $has_error = 1;
            $has_exception = 1;
        }
    };

    try { $out->flush };
    if (blessed($export_batch)) {
        $export_batch = state51::ExportBatch->load($export_batch_id);
        $export_batch->notes($out);
        $export_batch->update;
        unless ($export_batch->status eq "complete" && ($export_batch->service_status eq "transferred" || $export_batch->service_status eq "processing" || $export_batch->service_status eq "waiting") && scalar(@{ $export_batch->export_item })) {
            $has_error = 1;
            $comments .= "\n\nERROR: Export batch " . $export_batch->id . " is in status " . $export_batch->status . " transfer status " . $export_batch->service_status
                . "\n\nOUTPUT:$out";
        }
    }
    else {
        $comments .= "\n\nERROR - DID NOT GENERATE EXPORT BATCH. OUTPUT:\n$out";
    }

    my ($calculated_status, $all_deliveries_status, $delivery_status)  = $self->_order_object->calculate_status;
    my $full_comments = sprintf("%s Automatic delivery for order %s to %s executed at %s has finished with messages below\n%s\n\nOrder delivery status: %s\n",
        $self->can('date') ? $self->date:'unknown date', $self->order,
        $self->_delivery_point->tla, $my_hostname, $comments, $all_deliveries_status);

    eval {
        $self->update_ticket_status('open') if $has_error;
    };
    if ($@) {
        $self->log_message(level => "warn", message => ["Order object id:" . $self->order . ". DeliverOneDPForOrder: Error adding comment in RT: $@\n".$full_comments]);
    }
    print "\n---------------------------\n";
    print $log_fh "\n---------------------------\n" if $log_fh;
    print $full_comments;
    print $log_fh $full_comments if $log_fh;
    close $log_fh if $log_fh;

    # "empty" logfile from gupload is 100 bytes long, and pointless to upload
    if (-e $log_path && $export_batch && (-s $log_path != 100)) {
        eval {
            my $file = state51::ExportBatchLogFile->new(
                batch => $export_batch,
                file => $log_path,
            );
            $file->insert;
        };
        if ($@) {
            $self->log_message(level => "warn", message => ["Order object id:" . $self->order . " export batch id:" . $export_batch->id . ". Error storing export batch log: $@"]);
        };
    }

    my $products_count = $export_batch ? scalar(@{$export_batch->export_item}) : 0;
    #if ($self->can_trigger_more_delivery && $needs_more && $products_count) {
    $work_to_do = state51::SQLHacks::are_all_dds_finished_for_this_dp($self->_order_object, $self->_delivery_point);
    if ($work_to_do) {
        $self->log_message(level => "info", message => ["DeliverOneDPForOrder found more work to do for order object id:" . $self->order]);

        $self->run_job(__PACKAGE__, order => $self->order,
                       delivery_point_id => $self->delivery_point_id,
                       self_started => 1,
                       async => 1
                      );
    }
    elsif ('complete' eq $calculated_status) {
        $self->_order_object->complete_order('automatic delivery script at end '.$self->freeze." ".
            encode_json([$calculated_status, $all_deliveries_status, $delivery_status]), $my_hostname);
    }

    return $has_exception;
}

sub lockname {
    my ($self) = @_;

    if ($self->_delivery_point->serialize_deliveries) {
        return "order_deliver_to_".$_[0]->delivery_point_id;
    }
    return ''; # no lock
}

sub claim_dds_lockname {
    "claim_order_deliver_".$_[0]->order."_to_".$_[0]->delivery_point_id;
}

sub cannot_write_log {
    my ($self, $log_path) = @_;
    eval {
        my $error = sprintf("Export failing to %s on %s (%s). Cannot write log file: %s ",
            $self->_delivery_point->tla, (split qr/\./, Sys::Hostname::hostname)[0],
            $self->_delivery_point->ed_url, $log_path);
        warn "DeliverOneDPForOrder: " . $error;
        $self->update_ticket_status('open');
    };
    if ($@) {
        $self->log_message(level => "warn", message => ["Order object id:" . $self->order . ". DeliverOneDPForOrder cannot_write_log: $@"]);
    }
    return 1;
}

sub get_log_path {
    my $self = shift;
    my $log_dir = $self->_delivery_point->export_path . '/log';
    mkdir $log_dir unless -d $log_dir;
    my $log_path =
      $log_dir . '/' . $self->_order_object->id . '_' . $self->uuid;

    return $log_path;
}

sub get_log_fh {
    my ($self, $log_path) = @_;
    open my $log_fh, '>>', $log_path
      or return undef;

    chmod 0664, $log_path;

    return $log_fh;
}

with qw/
    state51::Script::Helper::InteractiveSuccess

    state51::Mixin::Loggable
    state51::Mixin::HandleOrdersOnJobs
    state51::Mixin::JobSignature
    state51::Mixin::RunsAsyncJobs
    state51::Mixin::JobLogger
    state51::Mixin::HasLocks
    state51::Mixin::LockedJob
/;

#
__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;
