package state51::Job::DetectFormatLanguage;
use MooseX::Types::Moose qw/ Int /;
use Moose;
use LWP::Simple;
use JSON qw/decode_json/;

use MooseX::ClassAttribute;
extends 'state51::Job';

class_has documentation => (
    is => 'ro',
    isa => 'Str',
    default => 'Given a releaseformat id we pass the label copy to the google translate api to autodetect the language. This is only enabled for certain asset controllers. If successful we update the format language reference.'
);

has 'release_format_id' => (
    is            => 'ro',
    isa           => Int,
    documentation => "The state51::ReleaseFormat ID that we are acting on",
);

sub exchange_name { 'detectformatlanguage' }

sub detect_langugage_for_format {
    my ($self, $format) = @_;

    my $api_key = $ENV{'GOOGLE_API_KEY'} or die "No GOOGLE_API_KEY env set";

    my $format_title = $format->title_with_version || '';
    my $track_titles = '';
    my @tracks;
    foreach my $track ( @{ $format->ordered_tracks } ) {
        $track_titles .= " " . $track->label_copy_with_mix;
        push @tracks, $track;
    }
    my $titles = $format_title . " " . $track_titles;

    my $base_url =
"https://www.googleapis.com/language/translate/v2/detect?key=$api_key&format=text&prettyprint=false&q=";

# The URL for the GET request, including parameters, must be less than 2K characters.
    my $sample_text = substr( $titles, 0, ( 2000 - length($base_url) ) );

    $sample_text =~ s/\%//g;
    $sample_text =~ s/\?//g;
    $sample_text =~ s!/!!g;

    # sometimes this get fails for no reason, call again and it's fine
    my $response_doc = get( $base_url . $sample_text );

    if ($response_doc) {
        my $response = decode_json($response_doc);

        my $language = $response->{data}->{detections}->[0]->[0]->{language};
        my $confidence =
          $response->{data}->{detections}->[0]->[0]->{confidence};
        my $reliable = $response->{data}->{detections}->[0]->[0]->{isReliable};

        return [ $language, $confidence, $reliable ];
    }
    else {
        $self->log_message(
            message => [
                    "Language detection failed: call to google API failed <<"
                  . $base_url
                  . $sample_text . ">>"
            ],
            level => "info",
        );
    }

    return undef;
}

sub run {
    my ($self) = @_;
    state51::Base->connect;

    my $format = state51::ReleaseFormat->load( $self->release_format_id )
      or die "Can't load ReleaseFormat: " . $self->release_format_id;

    # don't spend money for tests
    return 1 if state51::Base->devmode();

    # don't spend money for everyone, just select orgs
    return 1
      unless (
        ( $format->asset_controller )
        and (  $format->asset_controller->name eq 'The state51 Conspiracy'
            || $format->asset_controller->name eq 'Bonsound'
            || $format->asset_controller->name eq 'The Recording Consort S.L' )
      );

    # don't spend money if we've already got a language
    return if $format->language();

    my %all_languages;
    foreach my $language ( @{ state51::Language->fetch_whole_class() } ) {
        $all_languages{ $language->iso_code2 } = $language;
    }

    my $lowest_reasonable_confidence = 0.05;

    my $response = $self->detect_langugage_for_format($format);

    if ($response) {

        #my ($language, $confidence, $reliable) = @{$response};
        my ( $language, $confidence ) = @{$response};

        if (    ( $language ne 'und' )
            and ( $confidence > $lowest_reasonable_confidence ) )
        {
            if ( $all_languages{$language} ) {
                $format->language( $all_languages{$language} );
                $format->update();
                $self->log_message(
                    message => [
                        "Language detection succeeded for "
                          . $self->release_format_id
                    ],
                    level => "info",
                );
            }
            else {
                warn "google language API returned an unsupported language : <<"
                  . $language . ">>";
            }
        }
    }

    return 1;
}

__PACKAGE__->meta->make_immutable;
1;
