package state51::Job::ElasticSearchReindex;
use Moose;
use MooseX::Storage;
use MooseX::Types::Moose qw/ Int Str Bool Undef/;
use state51::Logger;
use Search::Elasticsearch;

extends 'state51::Job';

sub exchange_name { 'elasticsearchreindex' }

has source_index => (
    isa      => Str,
    is       => 'ro',
    required => 1,
    documentation =>
      'The name of an elasticsearch index filled with records you want to move.'
);

has target_index => (
    isa      => Str,
    is       => 'ro',
    required => 1,
    documentation =>
      'The name of a fancy rollover index to want to reindex records into.'
);

has delete_source_when_complete => (
    isa     => Bool,
    is      => 'ro',
    default => 0,
    documentation =>
'If this is true, the Job will wait until all records have reindex, check the task completed OK, then delete the source index.'
);

has expected_record_count => (
    isa => Int | Undef,
    is  => 'ro',
    documentation =>
'If you already know how many records you expect to move, pass it in. Otherwise we count the records in the source index.'
);

has _es => (
    is      => 'ro',
    isa     => 'Object',
    lazy    => 1,
    default => sub {
        return Search::Elasticsearch->new(
            nodes           => 'es02:9200',
            cxn_pool        => 'Sniff',
            cxn             => 'LWP',
            request_timeout => 120,
        );
    },
    traits => [qw/ NoGetopt DoNotSerialize /],
);

sub run {
    my ($self) = @_;

    # we don't use the schema, but we do use the mysql connection for locking
    state51::Base->connect;

    my $lockname = $self->lockname;
    $self->do_whilst_locked(
        $lockname,
        sub {
            my $logger = state51::Logger->new();

            my ($source_index, $target_index) =
              ($self->source_index, $self->target_index);
            my $es                   = $self->_es();
            my $source_records_count = $self->expected_record_count;
            if (not defined $source_records_count) {
                my $es_query = $es->count(index => $source_index);
                $source_records_count = $es_query->{count};
            }

            $logger->message(
                level => "info",
                message =>
                  ["Going to reindex from $source_index to $target_index"]
            );

            my $reindex_body = {
                "source" => {
                    "index" => $source_index
                },
                "dest" => {
                    "index" => $target_index
                }
            };

            # you can't wait for completion, the connection times out
            my $response = $es->reindex(
                body                => $reindex_body,
                requests_per_second => 10000,
                wait_for_completion => 0,
                timeout             => '2m'
            );

            my $task_id = $response->{task};

            $logger->message(
                level   => "info",
                message => ["reindex task id : $task_id"],
            );

            if (not $self->delete_source_when_complete()) {
                return undef;
            }

            my $completed = 0;
            while ($completed eq 0) {
                sleep(20);

                $response = $es->tasks->get(task_id => $task_id);
                $completed = $response->{completed};
                my $created = $response->{task}->{status}->{created};
                $logger->message(
                    level => "info",
                    message =>
                      ["created so far: $created of $source_records_count"],
                );
            }

            # final check
            sleep(20);
            $response = $es->tasks->get(task_id => $task_id);
            my $created = $response->{task}->{status}->{created};
            $logger->message(
                level   => "info",
                message => ["created: $created of $source_records_count"],
            );
            if ($created != $source_records_count) {
                $logger->message(
                    level   => "fatal",
                    message => [
"created != source_records_count $created $source_records_count"
                    ],
                );
            }

            $es->indices->delete(index => $source_index);
            $logger->message(
                level   => "info",
                message => ["index $source_index deleted."],
            );
        }
      )
      || do {
        if ($self->interactive) { warn("Already locked: $lockname") }
      };
    return 1;
}

sub lockname {
    my ($self) = @_;

    my $lockname = join('_', 'reindex', $self->target_index);
    $lockname =~ s/incoming\-spotify\-//;
    return $lockname;
}

with qw/
  state51::Script
  Log::Message::Structured
  Log::Message::Structured::Stringify::AsJSON
  state51::Mixin::HasLocks
  /;

__PACKAGE__->meta->make_immutable;
1;
