package state51::Job::ManageContract;
use Moose;
use MooseX::Storage;
use MooseX::Types::Moose qw/ Maybe Int Str Bool Undef/;
use Moose::Util::TypeConstraints;
use state51::Logger;
extends 'state51::Job';

sub exchange_name { 'managecontract' }

has 'signed_by_id' => (
    isa      => Int,
    is       => 'ro',
    required => 0,
    documentation =>
      'The instance id of the Organisation signing an Offered Contract.'
);

has 'hellosign_signature_request_id' => (
    isa      => 'Maybe[Str]',
    required => 0,
    is       => 'ro',
);

has 'file_id' => (
    isa           => Int,
    is            => 'ro',
    required      => 0,
    documentation => 'The instance id of the signed Offered Contract file.'
);

has 'action' => (
    is  => 'ro',
    isa => enum( [qw/ assign withdraw enliven/] ),
    documentation =>
"The action we are taking on the state51::Contract; assign, withdraw or enliven",
);

sub run {
    my ($self) = @_;
    state51::Base->connect;
    my $signer = state51::Organisation->load( $self->signed_by_id );
    my $contract;

    if ( $self->action eq 'assign' ) {
        $contract = state51::Contract->fetch_one(
            status    => 'Offered',
            signed_by => $signer
        );
        $contract->hellosign_signature_request_id(
            $self->hellosign_signature_request_id );
        $contract->update();
    }
    elsif ( $self->action eq 'enliven' ) {
        $contract =
          state51::Contract->fetch_one( hellosign_signature_request_id =>
              $self->hellosign_signature_request_id );
        if ( $contract->status eq 'Offered' ) {
            $self->log_message(
                level => "info",
                message =>
                  [ "Contract id " . $contract->id . " is being made live." ]
            );
            $contract->make_live();
            if ( $self->file_id ) {
                my $file = state51::File->load_polymorphic( $self->file_id );
                $contract->original_contract_file($file);
            }
            $contract->update();
        }
        else {
            $self->log_message(
                level   => "warn",
                message => [
                        "Contract id "
                      . $contract->id
                      . " cannot be made live as it is not currently 'offered'"
                ]
            );
        }
    }
    elsif ( $self->action eq 'withdraw' ) {
        $contract =
          state51::Contract->fetch_one( hellosign_signature_request_id =>
              $self->hellosign_signature_request_id );
        eval { $contract->_set_status('Withdrawn'); };
        if ($@) {
            $self->log_message(
                level   => "warn",
                message => [
                        "Contract id "
                      . $contract->id
                      . " cannot be withdrawn as it is not currently 'Offered'"
                ]
            );
        }
    }

    return 1;
}

with qw/
    state51::Mixin::JobLogger
/;

__PACKAGE__->meta->make_immutable;
1;
