package state51::Job::ManageDisplayArtistRole;
use Moose;
use MooseX::Types::Moose qw/ Str Int/;
use Moose::Util::TypeConstraints;
use MooseX::ClassAttribute;
use state51::DisplayArtistRole;
use Time::HiRes qw(gettimeofday tv_interval);
extends 'state51::Job';

class_has documentation => (
    is => 'ro',
    isa => 'Str',
    default => 'Allows you to manage the display artist role for a particular artist on a particular asset.'
);

has 'artist_id' => (
    isa           => Int,
    is            => 'ro',
    documentation => 'The instance id of the Artist.',
    required => 1,
);

has 'asset_id' => (
    isa           => Int,
    is            => 'ro',
    documentation => 'The instance id of a Track or Releaseformat that the artist currently appears on.',
    required => 1,
);

has 'displayartistrole' => (
    isa           => Str,
    is            => 'ro',
    documentation => 'The DisplayArtistRole that the artist will have on the asset. Must be one of MainArtist, Artist or FeaturedArtist.',
    required => 1,
);

has 'action' => (
    is  => 'ro',
    isa => enum([qw/ update /]),
    documentation =>
"The action we are taking on the display artist role, currently only update is supported",
);

sub exchange_name {
    'manage_displayartistrole'
}

sub suppress_console_display { 1 }

sub run {
    my ($self) = @_;

    my $t0 = [gettimeofday];
    state51::Base->connect;
    my $elapsed = tv_interval ( $t0 );

    $self->logger->message(
        level   => "info",
        message => [
            state51::Logger::Event::Generic->new(
                message => "ManageDAR Connect " . $elapsed
            )
        ]
    );

    $self->execute_action();

    return 1;
}

sub execute_action {
    my ($self) = @_;

    if ($self->action() eq 'update') {
        $self->log_message(
            "Job::ManageDisplayArtistRole called to update " . $self->artist_id. " on ". $self->asset_id. " to have display artist role ". $self->displayartistrole);

        my $artist = state51::Artist->load($self->artist_id);
        my $asset_id = $self->asset_id();
        if ($asset_id =~ m/0032|0087/) {
            $asset_id = substr($asset_id, 0, -4);
        }
        my $asset = state51::Base->load_polymorphic($asset_id);

        state51::DisplayArtistRole::set_displayartistrole_for_asset($asset, $artist, $self->displayartistrole);
    }
}

with qw/
    state51::Mixin::JobLogger
    state51::Mixin::HasLogger
/;

__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;
