package state51::Job::RaiseFlagsForQAAdvisor;
use state51::Base;
use MooseX::Types::Moose qw/ ArrayRef Str Int HashRef /;
use MooseX::Types::Varchar qw/ Varchar /;
use Try::Tiny;
use Module::Find qw/ useall /;
use Carp;
use Moose;
extends 'state51::Job';

my @qa_advisor_rules;

BEGIN {
    @qa_advisor_rules = useall state51::QAAdvisorRule;
}

has 'advisor_class' => (
    isa => Str,
    is  => 'ro',
    documentation =>
'QAAdvisorRule class to raise flags for, e.g. "state51::QAAdvisorRule::ITMSStyle"'
);

has 'format_oid' => (
    isa     => Int,
    is      => 'ro',
    default => sub { [] },
    documentation =>
'ReleaseFormat OID to raise QA advisor flags against. This is a full instance id including the class id on the end.'
);

has 'flags_to_raise' => (
    isa => ArrayRef [HashRef],
    is => 'ro',
    default => sub { [] },
    documentation =>
'Array of hashrefs that can be used to instantiate QANotes. Each ref must contain "flag",'
      . 'which is the flag id to raise. It can optionally contain track and volume,'
      . 'to relate the flag to a specific track. Note that this is the complete set of flags '
      . 'for this advisory class that should be open. Any other flags currently open for this class '
      . 'will be closed.'
);

sub exchange_name {
    'raiseflagsforqaadvisor'
}

has watchlist_match_detail => (
    is       => 'rw',
    isa      => Varchar [600],
    required => 0,
    default  => undef,
    documentation =>
'Holds a user facing message detailing why a watchlist QA flag was raised',
);

sub suppress_console_display { 1 }

sub run {
    my ($self) = @_;
    state51::Base->connect;

    $self->raise_flags_for_advisor($self->advisor_class, $self->format_oid,
        $self->flags_to_raise);

    return 1;
}

sub raise_flags_for_advisor {
    my ($self, $advisor_class, $format_id, $flat_flags_to_raise) = @_;

    my $format  = state51::ReleaseFormat->load($format_id);
    my $advisor = $advisor_class->new();

    my @flag_type_to_raise;
    foreach my $flag_to_raise (@{$flat_flags_to_raise}) {
        my %args;
        $args{flag}    = $flag_to_raise->{flag};
        $args{advisor} = $advisor;

        if ($flag_to_raise->{track}) {
            $args{track}  = $flag_to_raise->{track};
            $args{volume} = $flag_to_raise->{volume};
        }

        push @flag_type_to_raise, state51::QANote->new(%args);
    }

    my @current_open_flags = state51::QAFlag->fetch(release_format => $format);
    my %remaining_flags;

    if (@flag_type_to_raise) {
        foreach my $flag_obj (@flag_type_to_raise) {
            my $found_flag = $self->ensure_is_flagged($flag_obj, $format);

            $remaining_flags{$found_flag->id} = 1;
        }
    }

    foreach my $flag (@current_open_flags) {
        if (  !$remaining_flags{$flag->id}
            && $advisor->can_i_raise($flag->qa_flag_type->qa_advice_id))
        {
            $flag->erase();
        }
    }

    $format->qa_flags_run(state51::Base->dbnow);
    $format->update;
}

sub ensure_is_flagged {
    my ($self, $flag_obj, $format) = @_;

    my $flag_type = $flag_obj->flag;
    my $type = state51::QAFlagType->fetch_one(qa_advice_id => $flag_type);
    my $found_flag;

    if ($flag_obj->is_track) {

        # doesn't matter if it's closed
        ($found_flag) = state51::QAFlag::Track->fetch(
            release_format => $format,
            qa_flag_type   => $type,
            volume_part    => $flag_obj->volume,
            track_number   => $flag_obj->track,
        );
        if (not $found_flag) {
            $found_flag = state51::QAFlag::Track->new(
                release_format => $format,
                qa_flag_type   => $type,
                volume_part    => $flag_obj->volume,
                track_number   => $flag_obj->track,
            );
            $found_flag->insert();
        }
    }
    else {
        # doesn't matter if it's closed
        ($found_flag) = state51::QAFlag->fetch(
            release_format => $format,
            qa_flag_type   => $type,
        );
        if (not $found_flag) {
            $found_flag = state51::QAFlag->new(
                release_format => $format,
                qa_flag_type   => $type
            );
            $found_flag->insert();
        }

        if ($self->watchlist_match_detail) {
            $found_flag->watchlist_match_detail($self->watchlist_match_detail);
            $found_flag->update();
        }
    }
    return $found_flag;
}

__PACKAGE__->meta->make_immutable;
__PACKAGE__->new_with_options->run unless caller;
1;
