package state51::Job::RunEncode;
use Moose;
use MooseX::Types::Moose qw/ Int /;
use state51::Transformers;
use Sys::Hostname qw/ hostname /;
use Try::Tiny;
use File::Temp qw/ tempfile /;
use MooseX::ClassAttribute;
extends 'state51::Job';

class_has documentation => (
    is => 'ro',
    isa => 'Str',
    default => 'This job accepts an instance id of a pending state51::FileTransform and attempts to encode it.'
);

sub exchange_name {
    my ($self) = @_;

    my $transform = state51::FileTransform->load($self->id);
    my $priority  = $transform->priority || 0;
    my $family    = $transform->encode->family || '';

    return "run_encode_mqa" if $family eq 'MQA';
    return "run_encode_background" if $priority < 50;
    return "run_encode_catalogue"  if $priority < 100;
    return "run_encode_frontline"  if $priority < 200;
    return "run_encode_premium"    if $priority == 200;
    return "run_encode_boosted";
}

sub suppress_console_display { 1 }

has id => (
    isa => Int,
    is => 'ro',
    required => 1,
);


sub run {
    my $self = shift;
    srand();
    state51::Base->connect;

    $ENV{FILE_CACHE_DIR} || die "No FILE_CACHE_DIR env var defined";
    # tempfile will croak if it cannot create the file.
    (undef, my $check_writeable) = tempfile( DIR => $ENV{FILE_CACHE_DIR} );
    unlink $check_writeable;

    # see also Transformers.pm
    my %permitted_families = (
        mp3tag        => 1,
        mp3           => 1,
        'mplayer wav' => 1,
        aac           => 1,
        aac_plus      => 1,
        flac          => 1,
        wav           => 1,
        vorbis        => 1,
        aactom4a      => 1,
        am            => 1,
        alac          => 1,
        fingerprin    => 1,
        MQA           => 1,
        chromaprint   => 1,
        mir_stats     => 1,
        class_spch    => 1,
    );


    my $xform;
    try {
        $xform = state51::FileTransform->load($self->id);
        if ($permitted_families{$xform->encode->family}) {
            $xform->claim(encoder_hostname => hostname());
        }
        else {
            $self->log_message("wrong family, not touching ".$xform->id, "info");
            $xform = undef;
        }
    }
    catch {
        $self->log_message("failed to claim: ".$xform->id." $_", "trace");
        $xform = undef;
    };

    if ($xform) {
        try {
            # Get it downloaded before we begin.
            $xform->master->local_file_path;
            $self->log_message("actually running transform ".$xform->id, "trace");
            state51::Transformers::run_transform($xform);
        }
        catch {
            $self->log_message("transform failed: ".$xform->id." $_", "warn");
            $xform->status('failed');
            $xform->status_cause($_);
            $xform->update;
        };

        # Before letting the file object fall out of scope and losing
        # the locally cached copy, we should run any resulting
        # transforms.
    }
    state51::Base->unload_all;
    return;
}

with qw/
    state51::Mixin::JobLogger
/;

__PACKAGE__->meta->make_immutable;
1;

