package state51::Job::RunVPD;
use Moose;
use MooseX::Storage;
with Storage(format => "JSON");

use MooseX::Types::Moose qw/ Int /;
use File::Temp qw/ tempfile /;
use LWP::Simple;
use JSON;
use Time::HiRes qw{gettimeofday tv_interval};
use File::Slurp;

# given a track id, this Job finds an associated ripper wav files audio file
# submits it to the VPD service
# stores the result in a JSON file and associates it with the track
# and also promotes the two virtual stem files to be audio files
# and associates them with the track.

extends 'state51::Job';

sub exchange_name {
    my ($self) = @_;

    return "run_vpd";
}

sub suppress_console_display { 1 }

has track_id => (
    isa      => Int,
    is       => 'ro',
    required => 1,
);

sub run {
    my ($self) = @_;
    state51::Base->connect;

    $self->log_message("RunVPD Requested for track " . $self->track_id);

    my $rwf = state51::Encoding->load(63773160174);

    my $track = state51::Trackv2->load($self->track_id);

    my ($source_audio) = state51::File::Audio->fetch(
        track    => $track,
        status   => 'live',
        stored   => 1,
        encoding => $rwf,
    );

    return unless $source_audio;

    # FIXME: install a GPU somewhere else eh
    my $endpoint = 'http://arp.openimp.net:3000/';

    my $audio_url = $source_audio->external_url;

    $self->log_message("Starting Vocal Performance Detection on " . $track->id);
    my $t0            = [gettimeofday];
    my $json_response = get($endpoint . '?file=' . $audio_url);
    my $elapsed       = tv_interval($t0);
    $self->log_message("Vocal Performance Detection for "
          . $track->id
          . " took "
          . $elapsed
          . " seconds.");

    my $response = decode_json($json_response);

    my $vocal_virtual_stem = $response->{vocal_instance_id};
    $vocal_virtual_stem = $self->promote_file($vocal_virtual_stem, $track);
    my $accompaniment_virtual_stem = $response->{accompaniment_instance_id};
    $accompaniment_virtual_stem =
      $self->promote_file($accompaniment_virtual_stem, $track);

    # store the stats somewhere useful
    (my $fh, my $file_name) = tempfile(UNLINK => 1, TMPDIR => 1);
    write_file($file_name, $json_response);
    $fh->close();

    my $json_file = state51::File::Text->new_from_file($file_name);
    $json_file->write();
    $json_file->insert();

    $json_file->track($track);
    $json_file->tracks([$track]);
    $json_file->update();

    return;
}

# upgrade returned files to audio, correct encoding, reference track
sub promote_file {
    my ($self, $classless_instance_id, $track) = @_;

    my $virtual_stem_encoding = state51::Encoding->load(67280123890174);

    my $file = state51::File->load($classless_instance_id . '0053');
    my $file_audio = $file->transform_type('state51::File::Audio', 0);
    $file_audio->encoding($virtual_stem_encoding);
    $file_audio->track($track);
    $file_audio->tracks([$track]);
    $file_audio->recording($track->recording);
    $file_audio->update();
}

with qw/
  state51::Mixin::Loggable
  state51::Mixin::UnloadAfterJob
  state51::Mixin::HasLogger
  /;

__PACKAGE__->meta->make_immutable;
1;

